﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.Contracts.Interfaces;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Contracts;
using HeuristicLab.Security.Contracts.Interfaces;
using HeuristicLab.Hive.Server.Core.InternalInterfaces;
using System.ServiceModel;
using HeuristicLab.Hive.Server.DataAccess;
using HeuristicLab.Hive.Contracts.ResponseObjects;


namespace HeuristicLab.Hive.Server.Core {
  public class ServerConsoleFacade : IServerConsoleFacade {
    private ISlaveManager slaveManager = ServiceLocator.GetSlaveManager();

    private IJobManager jobManager = ServiceLocator.GetJobManager();

    private IHivePermissionManager secMan = ServiceLocator.GetHivePermissionManager();

    private IContextFactory contextFactory = ServiceLocator.GetContextFactory();

    public Guid sessionID = Guid.Empty;

    public ServerConsoleFacade() {
    }

    public Response Login(string username, string password) {
      Response resp = new Response();

      sessionID = secMan.Login(username, password);
      if (sessionID == Guid.Empty) {
        resp.StatusMessage = ResponseStatus.Login_LoginFaild;
      }
      return resp;
    }


    public ResponseList<SlaveDto> GetAllSlaves() {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessSlaves", sessionID, Guid.Empty);
        return slaveManager.GetAllSlaves();
      }
    }

    public ResponseList<SlaveGroupDto> GetAllSlaveGroups() {
      using (contextFactory.GetContext(false)) {
        //secMan.Authorize("AccessSlaveGroup", sessionID, Guid.Empty);
        return slaveManager.GetAllSlaveGroups();
      }
    }

    public ResponseList<UpTimeStatisticsDto> GetAllUpTimeStatistics() {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessStatistics", sessionID, Guid.Empty);
        return slaveManager.GetAllUpTimeStatistics();
      }
    }

    public ResponseObject<SlaveGroupDto> AddSlaveGroup(SlaveGroupDto slaveGroup) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("AddSlaveGroup", sessionID, Guid.Empty);
        return slaveManager.AddSlaveGroup(slaveGroup);
      }
    }

    public Response AddResourceToGroup(Guid slaveGroupId, ResourceDto resource) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("AddResource", sessionID, Guid.Empty);
        return slaveManager.AddResourceToGroup(slaveGroupId, resource);
      }
    }

    public Response DeleteResourceFromGroup(Guid slaveGroupId, Guid resourceId) {
      using (contextFactory.GetContext()) {
        return slaveManager.DeleteResourceFromGroup(slaveGroupId, resourceId);
      }
    }

    public ResponseList<JobDto> GetAllJobs() {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessJobs", sessionID, Guid.Empty);
        return jobManager.GetAllJobs();
      }
    }

    public ResponseList<JobDto> GetAllJobsWithFilter(JobState jobState, int offset, int count) {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessJobs", sessionID, Guid.Empty);
        return jobManager.GetAllJobsWithFilter(jobState, offset, count);
      }
    }

    public ResponseObject<JobDto> GetJobById(Guid jobId) {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessJobs", sessionID, jobId);
        return jobManager.GetJobById(jobId);
      }
    }

    public ResponseObject<JobDto> GetJobByIdWithDetails(Guid jobId) {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessJobs", sessionID, jobId);
        return jobManager.GetJobByIdWithDetails(jobId);
      }
    }

    public ResponseObject<JobDto> AddNewJob(SerializedJob job) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("AddJob", sessionID, job.JobInfo.Id);
        return jobManager.AddNewJob(job);
      }
    }

    //public ResponseObject<JobDto> GetLastJobResultOf(Guid jobId) {
    //  using (contextFactory.GetContext(false)) {
    //    secMan.Authorize("AccessJobResults", sessionID, jobId);
    //    return jobManager.GetLastJobResultOf(jobId);
    //  }
    //}

    public ResponseObject<SerializedJob> GetLastSerializedResult(Guid jobId, bool requested, bool snapshot) {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessJobResults", sessionID, jobId);
        return jobManager.GetLastSerializedResult(jobId, requested, snapshot);
      }
    }

    public ResponseObject<JobResultList> GetJobResults(IEnumerable<Guid> jobIds) {
      using (contextFactory.GetContext(false)) {
        //secMan.Authorize("AccessJobResults", sessionID, jobId); skip authorization
        return jobManager.GetJobResults(jobIds);
      }
    }

    //public Response RemoveJob(Guid jobId) {
    //  using (contextFactory.GetContext()) {
    //    secMan.Authorize("RemoveJob", sessionID, jobId);
    //    return jobManager.RemoveJob(jobId);
    //  }
    //}

    public Response RequestSnapshot(Guid jobId) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("AccessJobResults", sessionID, jobId);
        return jobManager.RequestSnapshot(jobId);
      }
    }

    public Response AbortJob(Guid jobId) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("AbortJob", sessionID, Guid.Empty);
        return jobManager.AbortJob(jobId);
      }
    }

    public ResponseObject<SlaveGroupDtoList> GetAllGroupsOfResource(Guid resourceId) {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessUserGroup", sessionID, Guid.Empty);
        return slaveManager.GetAllGroupsOfResource(resourceId);
      }
    }

    public Response DeleteSlaveGroup(Guid slaveGroupId) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("DeleteSlaveGroup", sessionID, Guid.Empty);
        return slaveManager.DeleteSlaveGroup(slaveGroupId);
      }
    }

    public ResponseList<ProjectDto> GetAllProjects() {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessProjects", sessionID, Guid.Empty);
        return jobManager.GetAllProjects();
      }
    }

    public Response CreateProject(ProjectDto project) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("CreateProjects", sessionID, Guid.Empty);
        return jobManager.CreateProject(project);
      }
    }

    public Response ChangeProject(ProjectDto project) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("ChangeProjects", sessionID, Guid.Empty);
        return jobManager.ChangeProject(project);
      }
    }

    public Response DeleteProject(Guid projectId) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("DeleteProjects", sessionID, projectId);
        return jobManager.DeleteProject(projectId);
      }
    }

    public ResponseList<JobDto> GetJobsByProject(Guid projectId) {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessJobs", sessionID, Guid.Empty);
        return jobManager.GetJobsByProject(projectId);
      }
    }

    public ResponseList<AppointmentDto> GetUptimeCalendarForResource(Guid guid) {
      using (contextFactory.GetContext(false)) {
        return slaveManager.GetUptimeCalendarForResource(guid);
      }
    }

    public Response SetUptimeCalendarForResource(Guid guid, IEnumerable<AppointmentDto> appointments, bool isForced) {
      using (contextFactory.GetContext()) {
        return slaveManager.SetUptimeCalendarForResource(guid, appointments, isForced);
      }
    }

    public ResponseObject<JobDto> AddJobWithGroupStrings(SerializedJob job, IEnumerable<string> resources) {
      using (contextFactory.GetContext()) {
        return jobManager.AddJobWithGroupStrings(job, resources);
      }
    }
  }
}
